#include <iostream>
#include <thread>
#include <vector>
#include <atomic>
#include <chrono>

using namespace std;

const long long NUM_THREADS = 16; // Try with 1 and check the time at the end of execution
const long long RANGE = 1000000;
atomic_llong result = 0;

void partial_sum(long long start, long long end) {
    for (long long i = start; i <= end; ++i) {
        result += i;
    }
}

int main() {
    auto start = chrono::high_resolution_clock::now();
    vector<thread> threads;
    long long chunk_size = RANGE / NUM_THREADS;
    
    for (long long i = 0; i < NUM_THREADS; ++i) {
        long long start = i * chunk_size + 1;
        long long end = (i + 1) * chunk_size;
        threads.emplace_back(partial_sum, start, end);
    }
    
    for (auto &t : threads) {
        t.join();
    }
    
    cout << "Total Sum: " << result << endl;
    auto stop = chrono::high_resolution_clock::now();
    auto duration = chrono::duration_cast<chrono::microseconds>(stop - start);
    cout << "Time taken by function: " << duration.count() << " microseconds" << endl;
    return 0;
}
